/*
 * test_w1.cc
 *
 *  Created on: Oct 20, 2010
 *      Author: lamikr
 */
#include <list>
#include <string>
#include <iostream>

#include <errno.h>
#include <stdlib.h>

#include <plp/log.h>

#include <plp/DeviceConfig.hh>
#include <plp/Device.hh>

#include <plp/str_util.h>

#include "Factory.hh"

using namespace w1;
using namespace std;
using namespace plp;

#define DEFAULT_READ_INTERVAL_SECONDS	600
#define DEFAULT_MAX_READ_COUNT		-1

int main(int argc, char** argv) {
	int				read_count_total;
	long				cnt_max_scans;
	long				read_interval_seconds;
	string				loc;
	Device 				*device;
	list<Device *>			device_list;
	list<Device *>::iterator	iter;

	// default values than can be overwritten with parameters
	loc	= "/tmp/w1data";
	read_interval_seconds	= DEFAULT_READ_INTERVAL_SECONDS;
	cnt_max_scans		= DEFAULT_MAX_READ_COUNT;
	if (argc > 1) {
		loc	= argv[1];
	}
	else {
		log_info("no parameter given using default values:\n");
		log_info("\ttest_w1_datalog_write %s %ld %ld\n\n", loc.c_str(), read_interval_seconds, cnt_max_scans);
		log_info("usage:\n\ttest_w1_datalog_write <result_save_path> <read_interval_seconds> <max_read_count>\n");
		log_info("\tmax_read_count == -1: read devices until application is terminated\n\n");
	}
	if (argc > 2) {
		parse_long(argv[2], &read_interval_seconds);
	}
	if (argc > 3) {
		parse_long(argv[3], &cnt_max_scans);
	}
	log_info("searching 1-wire devices\n");
	if (read_interval_seconds == DEFAULT_READ_INTERVAL_SECONDS) {
		log_info("\tdevice read interval: %ld seconds (default value)\n", read_interval_seconds);
	}
	else {
		log_info("\tdevice read interval: %ld seconds\n", read_interval_seconds);
	}
	log_info("\tdata save dir: %s\n", loc.c_str());
	if (cnt_max_scans == DEFAULT_MAX_READ_COUNT) {
		log_info("\tmax read count: %ld (default value, devices will be read until application is terminated)\n\n", cnt_max_scans);
	}
	else {
		log_info("\tmax read count: %ld\n\n", cnt_max_scans);
	}
	DeviceConfig::set_base_dir_name(loc);
	device_list		= Factory::get_device_list();
	read_count_total	= 0;
	if (device_list.size() > 0) {
		while(1) {
			read_count_total++;
			if ((cnt_max_scans != -1) &&
			    (read_count_total > cnt_max_scans)) {
				log_info("closing the application, max read count reached. (%ld)\n", cnt_max_scans);
				break;
			}
			for (iter = device_list.begin(); iter != device_list.end(); iter++) {
				device = (Device *)*iter;
				if (device->get_lifecycle_state() == LIFECYCLE_STATUS__AVAILABLE) {
					device->printout();
					sleep(1);
				}
				else {
					log_debug("device not available, %s.\n", device->get_id().c_str());
				}
			}
			sleep(read_interval_seconds);
		}
	}
	else {
		log_debug("could not find devices.\n");
	}
	log_debug("trying to start emptying list\n");
	while (device_list.empty() == false) {
		device	= device_list.back();
		device_list.pop_back();
		log_debug("calling delete\n");
		delete(device);
	}
	return 0;
}
